@@ -10,7 +10,10 @@
 #include <Wt/WContainerWidget>
 
 class Composer;
-class WTreeNode;
+
+namespace Wt {
+  class WTreeNode;
+}
 
 /**
  * \defgroup composerexample Composer example
@@ -30,7 +33,6 @@ private:
   Composer *composer_;
   WContainerWidget *details_;
 
-private slots:
   void send();
   void discard();
 };
