@@ -19,7 +19,14 @@ public:
   StdWidgetItemImpl(WWidgetItem *item);
   virtual ~StdWidgetItemImpl();
 
+  static const char *childrenResizeJS();
+  static const char *childrenGetPSJS();
+  static const char *secondResizeJS();
+  static const char *secondGetPSJS();
+
+  virtual const std::string id() const;
   virtual int minimumHeight() const;
+  virtual int minimumWidth() const;
 
   virtual void updateAddItem(WLayoutItem *);
   virtual void updateRemoveItem(WLayoutItem *);
@@ -28,9 +35,10 @@ public:
   virtual WLayoutItem *layoutItem() const;
   virtual void containerAddWidgets(WContainerWidget *container);
   virtual DomElement *createDomElement(bool fitWidth, bool fitHeight,
-				       int& additionalVerticalPadding,
 				       WApplication *app);
 
+  virtual void setHint(const std::string& name, const std::string& value);
+
 private:
   WWidgetItem *item_;
 };
