@@ -10,27 +10,47 @@
 #include <string>
 #include <map>
 
-#include "Configuration.h"
+#ifdef WT_THREADED
+#include <boost/thread.hpp>
+#endif // WT_THREADED
 
 namespace Wt {
 
 class SessionInfo;
+class WServer;
 
+/*
+ * A FastCGI relay server
+ */
 class Server
 {
 public:
-  Server(int argc, char *argv[]);
-  int main();
+  static bool bindUDStoStdin(const std::string& socketPath,
+			     Wt::WServer& server);
+
+  Server(WServer& wt, int argc, char *argv[]);
+  int run();
 
   static Server *instance;
 
   void handleSigChld();
-  void handleSigTerm();
+  void handleSignal(const char *signal);
 
 private:
+  WServer& wt_;
   int argc_;
   char **argv_;
-  Configuration conf_;
+
+  int childrenDied_;
+  volatile sig_atomic_t handleSigChld_;
+
+#ifdef WT_THREADED
+  // mutex to protect access to the sessions map
+  boost::recursive_mutex mutex_;
+#endif
+
+  void checkAndQueueSigChld();
+  void doHandleSigChld();
 
   void spawnSharedProcess();
   void execChild(bool debug, const std::string& extraArg);
@@ -41,6 +61,7 @@ private:
   bool getSessionFromQueryString(const std::string& uri,
 				 std::string& sessionId);
   void checkConfig();
+  bool writeToSocket(int socket, const unsigned char *buf, int bufsize);
 
   /*
    * For DedicatedProcess session policy
@@ -48,6 +69,9 @@ private:
   typedef std::map<std::string, SessionInfo *> SessionMap;
   SessionMap sessions_;
 
+  void handleRequestThreaded(int serverSocket);
+  void handleRequest(int serverSocket);
+
   /*
    * For SharedProcess session policy
    */
