@@ -32,16 +32,16 @@ namespace server {
 class WTHTTP_API StockReply : public Reply
 {
 public:
-  StockReply(const Request& request, status_type status,
+  StockReply(Request& request, status_type status,
 	     const Configuration& configuration);
 
-  StockReply(const Request& request, status_type status,
+  StockReply(Request& request, status_type status,
 	     std::string extraContent,
 	     const Configuration& configuration);
 
   virtual void reset(const Wt::EntryPoint *ep);
 
-  virtual void consumeData(Buffer::const_iterator begin,
+  virtual bool consumeData(Buffer::const_iterator begin,
 			   Buffer::const_iterator end,
 			   Request::State state);
 
