@@ -39,7 +39,7 @@ WT_DECLARE_WT_MEMBER
    var coordinates = computePopupPos(width, height);
    var w = window.open(url, "",
 	       "width=" + width + ",height=" + height +
-	       ",status=yes,location=yes,resizable=yes" +
+	       ",status=yes,location=yes,resizable=yes,scrollbars=yes" +
 	       ",left=" + coordinates.x + ",top=" + coordinates.y);
    w.opener = window;
 
