@@ -9,6 +9,7 @@
 
 #include <string>
 #include <vector>
+#include <deque>
 
 #if defined(WT_THREADED) || defined(WT_TARGET_JAVA)
 #define WT_BOOST_THREADS
@@ -25,6 +26,7 @@
 #include "TimeUtil.h"
 #include "WebRenderer.h"
 #include "WebRequest.h"
+#include "WebController.h"
 
 #include "Wt/WApplication"
 #include "Wt/WEnvironment"
@@ -97,7 +99,6 @@ public:
 
   void externalNotify(const WEvent::Impl& e);
   void notify(const WEvent& e);
-  void pushUpdates();
 
   void doRecursiveEventLoop();
 
@@ -175,10 +176,16 @@ public:
 
   class WT_API Handler {
   public:
+    enum LockOption {
+      NoLock,
+      TryLock,
+      TakeLock
+    };
+
     Handler();
     Handler(boost::shared_ptr<WebSession> session,
 	    WebRequest& request, WebResponse& response);
-    Handler(boost::shared_ptr<WebSession> session, bool takeLock);
+    Handler(boost::shared_ptr<WebSession> session, LockOption lockOption);
     Handler(WebSession *session);
     ~Handler();
 
@@ -210,6 +217,9 @@ public:
   private:
     void init();
 
+#ifndef WT_TARGET_JAVA
+    boost::shared_ptr<WebSession> sessionPtr_;
+#endif
 #ifdef WT_THREADED
     boost::mutex::scoped_lock lock_;
     boost::thread::id lockOwner_;
@@ -220,9 +230,6 @@ public:
     Handler *prevHandler_;
 
     WebSession *session_;
-#ifndef WT_TARGET_JAVA
-    boost::shared_ptr<WebSession> sessionPtr_;
-#endif
 
     WebRequest *request_;
     WebResponse *response_;
@@ -244,6 +251,7 @@ public:
   void setLoaded();
 
   void generateNewSessionId();
+  void queueEvent(const ApplicationEvent& event);
 
 private:
   void handleWebSocketRequest(Handler& handler);
@@ -257,11 +265,14 @@ private:
 
 #ifdef WT_BOOST_THREADS
   boost::mutex mutex_;
+  boost::mutex eventQueueMutex_;
   static boost::thread_specific_ptr<Handler> threadHandler_;
 #else
   static Handler *threadHandler_;
 #endif
 
+  std::deque<ApplicationEvent> eventQueue_;
+
   EntryPointType type_;
   std::string favicon_;
   State state_;
@@ -312,6 +323,7 @@ private:
 
   Handler *recursiveEventHandler_;
 
+  void pushUpdates();
   WResource *decodeResource(const std::string& resourceId);
   EventSignalBase *decodeSignal(const std::string& signalId,
 				bool checkExposed) const;
@@ -345,40 +357,48 @@ private:
   void flushBootStyleResponse();
   void changeInternalPath(const std::string& path, WebResponse *response);
 
+  void processQueuedEvents(WebSession::Handler& handler);
+  ApplicationEvent popQueuedEvent();
+
   friend class WebSocketMessage;
   friend class WebRenderer;
 };
 
 struct WEvent::Impl {
   WebSession::Handler *handler;
-#ifndef WT_CNOR
-  boost::function<void ()> function;
-#endif
+  WebResponse *response;
+  Function function;
   bool renderOnly;
 
   Impl(WebSession::Handler *aHandler, bool doRenderOnly = false)
     : handler(aHandler),
+      response(0),
       renderOnly(doRenderOnly)
   { }
 
-#ifndef WT_CNOR
-  Impl(WebSession::Handler *aHandler, const boost::function<void ()>& aFunction)
+  Impl(WebSession::Handler *aHandler, const Function& aFunction)
     : handler(aHandler),
+      response(0),
       function(aFunction),
       renderOnly(false)
   { }
-#endif
 
   Impl(const Impl& other)
     : handler(other.handler),
-#ifndef WT_CNOR
+      response(other.response),
       function(other.function),
-#endif // WT_CNOR
       renderOnly(other.renderOnly)
   { }
 
+  Impl(WebResponse *aResponse)
+    : handler(0),
+      response(aResponse),
+      renderOnly(true)
+  { }
+
   Impl()
-    : handler(0)
+    : handler(0),
+      response(0)
   { }
 };
 
