@@ -56,7 +56,7 @@ this.condCall = function(o, f, a) {
 this.buttons = 0;
 
 // button last released (for reporting in IE's click event)
-var lastButtonUp = 0;
+var lastButtonUp = 0, mouseDragging = false;
 
 // returns the button associated with the event (0 if none)
 this.button = function(e)
@@ -115,7 +115,18 @@ this.mouseDown = function(e) {
 
 this.mouseUp = function(e) {
   lastButtonUp = WT.button(e);
-  WT.buttons &= ~lastButtonUp;
+  setTimeout(function() {
+    mouseDragging = false;
+    WT.buttons &= ~lastButtonUp;
+  }, 5);
+};
+
+this.dragged = function(e) {
+  return mouseDragging;
+};
+
+this.drag = function(e) {
+  mouseDragging = true;
 };
 
 /**
@@ -255,9 +266,10 @@ this.initAjaxComm = function(url, handler) {
 
 	  clearTimeout(timer);
 
-	  if (good)
+	  if (good) {
+	    handled = true;
 	    handler(0, request.responseText, userData);
-	  else
+	  } else
 	    handler(1, null, userData);
 
 	  if (request) {
@@ -596,6 +608,9 @@ this.ajaxInternalPaths = function(basePath) {
 	if (href.charAt(0) != '/')
 	  href = '/' + href;
 	internalPath = href.substr(basePath.length);
+	if (internalPath.substr(0, 2) == "_=" &&
+	    basePath.charAt(basePath.length - 1) == '?')
+	  internalPath = '?' + internalPath;  /* eaten one too much */
       }
 
       if (internalPath.length == 0 || internalPath.charAt(0) != '/')
@@ -739,19 +754,35 @@ this.widgetCoordinates = function(obj, e) {
 // Get coordinates of (mouse) event relative to page origin.
 this.pageCoordinates = function(e) {
   if (!e) e = window.event;
+
   var posX = 0, posY = 0;
+
+  var target = e.target || e.srcElement;
+
+  // if this is an iframe, offset against the frame's position
+  if (target && (target.ownerDocument != document))
+    for (var i=0; i < window.frames.length; i++) {
+      if (target.ownerDocument == window.frames[i].document) {
+        try{
+          var rect = window.frames[i].frameElement.getBoundingClientRect();
+          posX = rect.left;
+          posY = rect.top;
+        }catch (e) {
+        }
+      }
+    }
   
   if (e.touches && e.touches[0]) {
     return WT.pageCoordinates(e.touches[0]);
   } else if (!WT.isIE && e.changedTouches && e.changedTouches[0]) {
-    posX = e.changedTouches[0].pageX;
-    posY = e.changedTouches[0].pageY;
+    posX += e.changedTouches[0].pageX;
+    posY += e.changedTouches[0].pageY;
   } else if (typeof e.pageX === 'number') {
-    posX = e.pageX; posY = e.pageY;
+    posX += e.pageX; posY = e.pageY;
   } else if (typeof e.clientX === 'number') {
-    posX = e.clientX + document.body.scrollLeft
+    posX += e.clientX + document.body.scrollLeft
       + document.documentElement.scrollLeft;
-    posY = e.clientY + document.body.scrollTop
+    posY += e.clientY + document.body.scrollTop
       + document.documentElement.scrollTop;
   }
 
@@ -779,7 +810,7 @@ this.wheelDelta = function(e) {
 };
 
 this.scrollIntoView = function(id) {
-  setTimeout(function() { 
+  setTimeout(function() {
       var hashI = id.indexOf('#');
       if (hashI != -1)
 	id = id.substr(hashI + 1);
@@ -905,6 +936,17 @@ this.isKeyPress = function(e) {
 
 var repeatT = null, repeatI = null;
 
+this.isDblClick = function(o, e) {
+  if (o.wtClickTimeout &&
+      Math.abs(o.wtE1.clientX - e.clientX) < 2 &&
+      Math.abs(o.wtE1.clientY - e.clientY) < 2) {
+      clearTimeout(o.wtClickTimeout);
+      o.wtClickTimeout = null; o.wtE1 = null;
+      return true;
+  } else
+      return false;
+};
+
 this.eventRepeat = function(fun, startDelay, repeatInterval) {
   WT.stopRepeat();
 
@@ -1160,16 +1202,10 @@ function mouseUp(e) {
 
 var captureInitialized = false;
 
-function initCapture() {
-  if (captureInitialized)
-    return;
-
-  captureInitialized = true;
-
-  var db = document.body;
-  if (db.addEventListener) {
-    db.addEventListener('mousemove', mouseMove, true);
-    db.addEventListener('mouseup', mouseUp, true);
+function attachMouseHandlers(el) {
+  if (el.addEventListener) {
+    el.addEventListener('mousemove', mouseMove, true);
+    el.addEventListener('mouseup', mouseUp, true);
 
     if (WT.isGecko) {
       window.addEventListener('mouseout', function(e) {
@@ -1179,17 +1215,37 @@ function initCapture() {
 			      }, true);
     }
   } else {
-    db.attachEvent('onmousemove', mouseMove);
-    db.attachEvent('onmouseup', mouseUp);
+    el.attachEvent('onmousemove', mouseMove);
+    el.attachEvent('onmouseup', mouseUp);
   }
 }
 
+function initCapture() {
+  if (captureInitialized)
+    return;
+
+  captureInitialized = true;
+
+  var db = document.body;
+  attachMouseHandlers(db);
+}
+
 this.capture = function(obj) {
   initCapture();
 
   if (captureElement && obj)
     return;
 
+  // attach to possible iframes
+  for (var i=0; i < window.frames.length; i++)
+    try{
+      if (! window.frames[i].document.body.hasMouseHandlers) {
+        attachMouseHandlers(window.frames[i].document.body);
+        window.frames[i].document.body.hasMouseHandlers = true;
+      }
+    }catch (e) {
+    }
+
   captureElement = obj;
 
   var db = document.body;
@@ -1357,6 +1413,42 @@ this.addStyleSheet = function(uri, media) {
   }
 };
 
+this.removeStyleSheet = function(uri) {
+  if ($('link[rel=stylesheet][href~="' + uri + '"]'))
+    $('link[rel=stylesheet][href~="' + uri + '"]').remove();
+  var sheets = document.styleSheets;
+  for (var i=0; i<sheets.length; ++i) {
+    var sheet = sheets[i];
+    j = 0;
+    if (sheet) {
+      var rule = null;
+      do {
+        try {
+          if (sheet.cssRules)
+            rule = sheet.cssRules[j]; // firefox
+          else
+            rule = sheet.rules[j];    // IE
+
+          if (rule && rule.cssText ===
+              "@import url(\"" + uri +  "\");") {
+            if (sheet.cssRules)
+              sheet.deleteRule(j);// firfox
+            else
+              sheet.removeRule(j);//IE
+            break; // only remove 1 rule !!!!
+          }
+        } catch (err) {
+          /*
+        * firefox security error 1000 when access a stylesheet.cssRules
+        * hosted from another domain
+        */
+        }
+        ++j;
+      } while(rule)
+    }
+  }
+};
+
 this.windowSize = function() {
   var x, y;
 
@@ -2055,6 +2147,8 @@ function initDragDrop() {
 }
 
 function dragStart(obj, e) {
+  if (e.ctrlKey || WT.button(e) > 1) //Ignore drags with rigth click.
+    return true;
   var t = WT.target(e);
   if (t) {
     /*
@@ -2601,14 +2695,16 @@ _$_$endif_$_();
       doJavaScript(msg);
 _$_$if_CATCH_ERROR_$_();
     } catch (e) {
-      var stack = null;
-
-_$_$if_SHOW_STACK_$_();
-      stack = e.stack || e.stacktrace;
-_$_$endif_$_();
-      alert("Wt internal error: " + e + ", code: " +  e.code
-	    + ", description: " + e.description
-	    + (stack ? (", stack:\n" + stack) : ""));
+      var stack = e.stack || e.stacktrace;
+      var description = e.description || e.message;
+      var err = { "exception_code": e.code,
+		  "exception_description": description,
+		  "exception_js": msg };
+      err.stack = stack;
+      sendError(err,
+		"Wt internal error; code: " +  e.code
+		+ ", description: " + description);
+      throw e;
     }
 _$_$endif_$_();
 
@@ -2706,6 +2802,22 @@ _$_$endif_$_();
 
 var updateTimeoutStart;
 
+function schedulePing() {
+  if (websocket.keepAlive)
+    clearInterval(websocket.keepAlive);
+
+  websocket.keepAlive = setInterval
+    (function() {
+      var ws = websocket.socket;
+      if (ws.readyState == 1)
+	ws.send('&signal=ping');
+      else {
+	clearInterval(websocket.keepAlive);
+	websocket.keepAlive = null;
+      }
+    }, _$_SERVER_PUSH_TIMEOUT_$_);
+}
+
 function scheduleUpdate() {
   if (quitted) {
     if (!quittedStr)
@@ -2796,27 +2908,17 @@ _$_$if_WEB_SOCKETS_$_();
 	     * motivate proxies to keep connections open, but we've never
 	     * seen a browser pinging us ?
 	     *
-	     * So, we ping pong ourselves. It costs virtually nothing.
+	     * So, we ping pong ourselves.
 	     */
 	    ws.send('&signal=ping'); // to get our first onmessage
 
-	    if (websocket.keepAlive)
-	      clearInterval(websocket.keepAlive);
-
-	    websocket.keepAlive = setInterval
-	      (function() {
-		if (ws.readyState == 1)
-		  ws.send('&signal=ping');
-		else {
-		  clearInterval(websocket.keepAlive);
-		  websocket.keepAlive = null;
-		}
-	      }, _$_SERVER_PUSH_TIMEOUT_$_);
+	    schedulePing();
 	  };
 	}
       }
 
       if (ws.readyState == 1) {
+	schedulePing();
 	sendUpdate();
 	return;
       }
@@ -2855,11 +2957,19 @@ function responseReceived(updateId, puzzle) {
 }
 
 var pageId = 0;
-function setPage(id)
-{
+function setPage(id) {
   pageId = id;
 }
 
+function sendError(err, errMsg) {
+  responsePending = comm.sendUpdate
+    ('request=jserror&err=' + encodeURIComponent(JSON.stringify(err)),
+     false, ackUpdateId, -1);
+_$_$if_SHOW_ERROR_$_();
+  alert(errMsg);
+_$_$endif_$_();
+}
+
 function sendUpdate() {
   if (self != window._$_APP_CLASS_$_) {
     quit(null);
@@ -2950,7 +3060,7 @@ function propagateSize(element, width, height) {
     element.wtHeight = height;
 
     if (width >= 0 && height >= 0)
-      emit(element, 'resized', width, height);
+      emit(element, 'resized', Math.round(width), Math.round(height));
   }
 }
 
@@ -3057,9 +3167,12 @@ function loadScript(uri, symbol, tries)
       if (t > 1) {
 	loadScript(uri, symbol, t - 1);
       } else {
-	alert('Fatal error: failed loading ' + uri);
+	var err = {
+	  "error-description" : 'Fatal error: failed loading ' + uri
+	};
+	sendError(err, err["error-description"]);
 	quit(null);
-      }      
+      }     
     }
   }
 
@@ -3219,6 +3332,12 @@ window.onunload = function()
   }
 };
 
+function setLocale(m)
+{	
+  if (WT.isIEMobile || m == '') return;
+  document.documentElement.lang = m
+}
+
 function setCloseMessage(m)
 {
   if (m && m != '') {
@@ -3239,6 +3358,7 @@ this._p_ = {
   loadScript : loadScript,
   onJsLoad : onJsLoad,
   setTitle : setTitle,
+  setLocale : setLocale,
   update : update,
   quit : quit,
   setSessionUrl : setSessionUrl,
